﻿import es
import gamethread
import langlib
import playerlib
import popuplib
import random
import repeat
import vecmath

lang_msg = es.import_addon("soccer_mod_2011")

lang_dir = "cstrike/addons/eventscripts/soccer_mod_2011/language/"

messages = langlib.Strings(lang_dir + "modules/training/messages_training.ini")
messages_common = langlib.Strings(lang_dir + "messages_common.ini")
texts = langlib.Strings(lang_dir + "modules/training/popup_training.ini")

popup = popuplib.create("sm_popup_admin_training_cannon")

settings = ["randomness", "fire_rate", "height", "power"]

# ******************** Load ********************
def load():
	es.set("sm_training_cannon_on", 0)
	es.set("sm_training_randomness", 1)
	es.set("sm_training_fire_rate", 2.5)
	es.set("sm_training_height", 1)
	es.set("sm_training_power", 10000.0)

	create_popup()

	sm_training_cannon = repeat.create("cannon", shoot, ())

	es.addons.registerSayFilter(filter)

# ******************** Create popup ********************
def create_popup():
	common = langlib.Strings(lang_dir + "popup_common.ini")

	popup.addline("SOCCER MOD %s" % (texts("title")))
	popup.addline(" ")

	values = ["position", "aim", "on", "off"]
	for i in range(0, len(values)):
		popup.addline("->%s. %s" % (i + 1, texts(values[i])))

	for i in range(0, len(settings)):
		popup.addline("->%s. %s: %s" % (i + 5, texts(settings[i]), es.ServerVar("sm_training_%s" % (settings[i]))))

	popup.addline(" ")
	popup.addline("->9. %s" % (common("back")))
	popup.addline(" ")
	popup.addline("->0. %s" % (common("close")))

	for i in range(1, 9):
		popup.select(i, popup_choice)

	popup.submenu(9, "sm_popup_admin_training")

# ******************** Events ********************
def es_map_start(event_var):
	sm_training_cannon = repeat.find("cannon")

	if sm_training_cannon.status() > 1:
		sm_training_cannon.stop()
		es.set("sm_training_cannon_on", 0)

def round_start(event_var):
	if es.ServerVar("soccer_mod_allowed") == 1:
		es.set("sm_training_ballid", es.getentityindex("func_physbox"))

def player_disconnect(event_var):
	if es.ServerVar("soccer_mod_allowed") == 1:
		if len(playerlib.getPlayerList("#human")) < 1:
			sm_training_cannon = repeat.find("cannon")

			if sm_training_cannon.status() > 1:
				sm_training_cannon.stop()
				es.set("sm_training_cannon_on", 0)

# ******************** Say filter ********************
def filter(userid, text, teamonly):
	if es.ServerVar("soccer_mod_allowed") == 1:
		steamid = es.getplayersteamid(userid)
		if es.exists("key", "sm_keygroup_set", steamid) == 1:
			text = text.strip('"')
			type = es.keygetvalue("sm_keygroup_set", steamid, "setting_type")
			if type == "training_randomness":
				set_value(userid, text, "randomness", 0, 1, 500, 5)
				return (0, None, 0)
			elif type == "training_fire_rate":
				set_value(userid, text, "fire_rate", 1, 0.5, 10, 6)
				return (0, None, 0)
			elif type == "training_height":
				set_value(userid, text, "height", 0, 1, 500, 7)
				return (0, None, 0)
			elif type == "training_power":
				set_value(userid, text, "power", 3, 0.001, 10000, 8)
				return (0, None, 0)
			else:
				return (userid, text, teamonly)
		else:
			return (userid, text, teamonly)
	else:
		return (userid, text, teamonly)

# ******************** Popup choice ********************
def popup_choice(userid, choice, popupid):
	tokens = {}
	player = playerlib.getPlayer(userid)
	tokens["name"] = player.name

	if es.ServerVar("sm_match_started") == 0:
		if choice == 1:
			es.set("cannon_x", 0)
			es.set("cannon_y", 0)
			es.set("cannon_z", 0)

			es.server.queuecmd("playerget viewcoord %s cannon_x cannon_y cannon_z" % (userid))

			gamethread.delayed(0.01, show_position)

			lang_msg.tell(messages, "position", tokens)

			popuplib.send("sm_popup_admin_training_cannon", userid)

		elif choice == 2:
			es.set("cannon_aim_x", 0)
			es.set("cannon_aim_y", 0)
			es.set("cannon_aim_z", 0)

			es.server.queuecmd("playerget viewcoord %s cannon_aim_x cannon_aim_y cannon_aim_z" % (userid))

			gamethread.delayed(0.01, show_aim_position)

			lang_msg.tell(messages, "aim_position", tokens)

			popuplib.send("sm_popup_admin_training_cannon", userid)

		elif choice == 3:
			if es.ServerVar("sm_training_ballid") > 0:
				sm_training_cannon = repeat.find("cannon")

				if sm_training_cannon.status() == 1:
					es.set("sm_training_cannon_on", 1)

					shoot()

					sm_training_cannon.start(float(es.ServerVar("sm_training_fire_rate")), 0)

					lang_msg.tell(messages, "on", tokens)
				else:
					es.tell(userid, "#green", messages("already_on", tokens, player.get("lang")))
			else:
				es.tell(userid, "#green", messages("no_ball", tokens, player.get("lang")))

			popuplib.send("sm_popup_admin_training_cannon", userid)

		elif choice == 4:
			sm_training_cannon = repeat.find("cannon")

			if sm_training_cannon.status() > 1:
				sm_training_cannon.stop()

				es.set("sm_training_cannon_on", 0)

				lang_msg.tell(messages, "off", tokens)
			else:
				es.tell(userid, "#green", messages("already_off", tokens, player.get("lang")))

			popuplib.send("sm_popup_admin_training_cannon", userid)

		else:
			if es.exists("key", "sm_keygroup_set", player.steamid) == 0:
				es.keycreate("sm_keygroup_set", player.steamid)

			es.keysetvalue("sm_keygroup_set", player.steamid, "setting_type", "training_%s" % (settings[choice - 5]))
			es.tell(userid, "#green", messages("type_%s" % (settings[choice - 5]), tokens, player.get("lang")))
	else:
		es.tell(userid, "#green", messages_common("no_option_match", tokens, player.get("lang")))
		popuplib.send("sm_popup_admin_training_cannon", userid)

# ******************** Functions ********************
def show_position():
	show_lines(es.ServerVar("cannon_x"), es.ServerVar("cannon_y"), es.ServerVar("cannon_z"))

def show_aim_position():
	show_lines(es.ServerVar("cannon_aim_x"), es.ServerVar("cannon_aim_y"), es.ServerVar("cannon_aim_z"))

def show_lines(x, y, z):
	model = es.precachemodel("sprites/crystal_beam1.vmt")

	pos1 = "%s,%s,%s" % ((x - 25), y, z)
	pos2 = "%s,%s,%s" % ((x + 25), y, z)
	es.server.queuecmd("es_effect beam %s %s %s %s 0 0 5 5 5 0 1 255 0 0 255 5" % (pos1, pos2, model, model))

	pos1 = "%s,%s,%s" % (x, (y - 25), z)
	pos2 = "%s,%s,%s" % (x, (y + 25), z)
	es.server.queuecmd("es_effect beam %s %s %s %s 0 0 5 5 5 0 1 255 0 0 255 5" % (pos1, pos2, model, model))

	pos1 = "%s,%s,%s" % (x, y, (z - 25))
	pos2 = "%s,%s,%s" % (x, y, (z + 25))
	es.server.queuecmd("es_effect beam %s %s %s %s 0 0 5 5 5 0 1 255 0 0 255 5" % (pos1, pos2, model, model))

def shoot():
	x = es.ServerVar("cannon_x")
	y = es.ServerVar("cannon_y")
	z = es.ServerVar("cannon_z")

	z += 15
	z += int(es.ServerVar("sm_training_height"))

	players = es.getUseridList()
	length = len(players)
	if length > 0:
		es.server.queuecmd("es_xfire %s func_physbox wake" % (players[random.randint(0, length - 1)]))

	es.server.queuecmd("es_xphysics active teleport %s %s,%s,%s" % (es.ServerVar("sm_training_ballid"), x, y, z))
	gamethread.delayed(0.1, push)

def push():
	aim_x = es.ServerVar("cannon_aim_x")
	aim_y = es.ServerVar("cannon_aim_y")
	aim_z = es.ServerVar("cannon_aim_z")

	randomness = int(es.ServerVar("sm_training_randomness"))
	if randomness > 0:
		aim_x = random.uniform(aim_x - randomness, aim_x + randomness)
		aim_y = random.uniform(aim_y - randomness, aim_y + randomness)
		aim_z = random.uniform(aim_z - randomness, aim_z + randomness)

	vector1 = vecmath.vector([aim_x, aim_y, aim_z])

	x = es.ServerVar("cannon_x")
	y = es.ServerVar("cannon_y")
	z = es.ServerVar("cannon_z")
	vector2 = vecmath.vector([x, y, z])

	es.set("sm_training_cannon_aim", (vector1 - vector2) * float(es.ServerVar("sm_training_power")))

	es.server.queuecmd("es_xphysics active setvelocity %s %s" % (es.ServerVar("sm_training_ballid"), es.ServerVar("sm_training_cannon_aim")))

def set_value(userid, text, msg, decimals, min, max, number):
	tokens = {}
	player = playerlib.getPlayer(userid)

	if text != "!cancel":
		error = messages("type_%s" % (msg), tokens, player.get("lang"))
		try:
			if decimals < 1:
				value = int(round(float(text), 0))
			else:
				value = round(float(text), decimals)

			if value >= min:
				if value <= max:
					es.set("sm_training_%s" % (msg), value)

					popuplib.find("sm_popup_admin_training_cannon").modline((number + 2), "->%s. %s: %s" % (number, texts(msg), value))

					if msg == "fire_rate":
						sm_training_cannon = repeat.find("cannon")

						if sm_training_cannon.status() > 1:
							sm_training_cannon.stop()
							es.doblock("soccer_mod_2011/modules/training/cannon/shoot")
							sm_training_cannon.start(float(es.ServerVar("sm_training_fire_rate")), 0)

					tokens["name"] = player.name
					tokens["value"] = value
					lang_msg.tell(messages, "set_%s" % (msg), tokens)

					es.keydelete("sm_keygroup_set", player.steamid)
					popuplib.send("sm_popup_admin_training_cannon", userid)
				else:
					es.tell(userid, "#green", error)
			else:
				es.tell(userid, "#green", error)
		except ValueError:
			es.tell(userid, "#green", error)
	else:
		es.keydelete("sm_keygroup_set", player.steamid)
		popuplib.send("sm_popup_admin_training_cannon", userid)

# ******************** Unload ********************
def unload():
	repeat.delete("cannon")

	es.addons.unregisterSayFilter(filter)

	popuplib.delete("sm_popup_admin_training_cannon")